/* abstractsortedmap.c generated by valac 0.36.11, the Vala compiler
 * generated from abstractsortedmap.vala, do not modify */

/* readonlysortedmap.vala
 *
 * Copyright (C) 2009-2011  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Maciej Piechotka <uzytkownik2@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>


#define GEE_TYPE_TRAVERSABLE (gee_traversable_get_type ())
#define GEE_TRAVERSABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_TRAVERSABLE, GeeTraversable))
#define GEE_IS_TRAVERSABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_TRAVERSABLE))
#define GEE_TRAVERSABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_TRAVERSABLE, GeeTraversableIface))

typedef struct _GeeTraversable GeeTraversable;
typedef struct _GeeTraversableIface GeeTraversableIface;

#define GEE_TRAVERSABLE_TYPE_STREAM (gee_traversable_stream_get_type ())

#define GEE_TYPE_LAZY (gee_lazy_get_type ())
#define GEE_LAZY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_LAZY, GeeLazy))
#define GEE_LAZY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_LAZY, GeeLazyClass))
#define GEE_IS_LAZY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_LAZY))
#define GEE_IS_LAZY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_LAZY))
#define GEE_LAZY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_LAZY, GeeLazyClass))

typedef struct _GeeLazy GeeLazy;
typedef struct _GeeLazyClass GeeLazyClass;

#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))

typedef struct _GeeIterator GeeIterator;
typedef struct _GeeIteratorIface GeeIteratorIface;

#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))

typedef struct _GeeIterable GeeIterable;
typedef struct _GeeIterableIface GeeIterableIface;

#define GEE_TYPE_MAP (gee_map_get_type ())
#define GEE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_MAP, GeeMap))
#define GEE_IS_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_MAP))
#define GEE_MAP_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_MAP, GeeMapIface))

typedef struct _GeeMap GeeMap;
typedef struct _GeeMapIface GeeMapIface;

#define GEE_TYPE_MAP_ITERATOR (gee_map_iterator_get_type ())
#define GEE_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_MAP_ITERATOR, GeeMapIterator))
#define GEE_IS_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_MAP_ITERATOR))
#define GEE_MAP_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_MAP_ITERATOR, GeeMapIteratorIface))

typedef struct _GeeMapIterator GeeMapIterator;
typedef struct _GeeMapIteratorIface GeeMapIteratorIface;

#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))

typedef struct _GeeCollection GeeCollection;
typedef struct _GeeCollectionIface GeeCollectionIface;

#define GEE_TYPE_SET (gee_set_get_type ())
#define GEE_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_SET, GeeSet))
#define GEE_IS_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_SET))
#define GEE_SET_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_SET, GeeSetIface))

typedef struct _GeeSet GeeSet;
typedef struct _GeeSetIface GeeSetIface;

#define GEE_MAP_TYPE_ENTRY (gee_map_entry_get_type ())
#define GEE_MAP_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_MAP_TYPE_ENTRY, GeeMapEntry))
#define GEE_MAP_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_MAP_TYPE_ENTRY, GeeMapEntryClass))
#define GEE_MAP_IS_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_MAP_TYPE_ENTRY))
#define GEE_MAP_IS_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_MAP_TYPE_ENTRY))
#define GEE_MAP_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_MAP_TYPE_ENTRY, GeeMapEntryClass))

typedef struct _GeeMapEntry GeeMapEntry;
typedef struct _GeeMapEntryClass GeeMapEntryClass;

#define GEE_TYPE_ABSTRACT_MAP (gee_abstract_map_get_type ())
#define GEE_ABSTRACT_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_MAP, GeeAbstractMap))
#define GEE_ABSTRACT_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_MAP, GeeAbstractMapClass))
#define GEE_IS_ABSTRACT_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_MAP))
#define GEE_IS_ABSTRACT_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_MAP))
#define GEE_ABSTRACT_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_MAP, GeeAbstractMapClass))

typedef struct _GeeAbstractMap GeeAbstractMap;
typedef struct _GeeAbstractMapClass GeeAbstractMapClass;
typedef struct _GeeAbstractMapPrivate GeeAbstractMapPrivate;

#define GEE_TYPE_SORTED_MAP (gee_sorted_map_get_type ())
#define GEE_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_SORTED_MAP, GeeSortedMap))
#define GEE_IS_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_SORTED_MAP))
#define GEE_SORTED_MAP_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_SORTED_MAP, GeeSortedMapIface))

typedef struct _GeeSortedMap GeeSortedMap;
typedef struct _GeeSortedMapIface GeeSortedMapIface;

#define GEE_TYPE_SORTED_SET (gee_sorted_set_get_type ())
#define GEE_SORTED_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_SORTED_SET, GeeSortedSet))
#define GEE_IS_SORTED_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_SORTED_SET))
#define GEE_SORTED_SET_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_SORTED_SET, GeeSortedSetIface))

typedef struct _GeeSortedSet GeeSortedSet;
typedef struct _GeeSortedSetIface GeeSortedSetIface;

#define GEE_TYPE_ABSTRACT_SORTED_MAP (gee_abstract_sorted_map_get_type ())
#define GEE_ABSTRACT_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_SORTED_MAP, GeeAbstractSortedMap))
#define GEE_ABSTRACT_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_SORTED_MAP, GeeAbstractSortedMapClass))
#define GEE_IS_ABSTRACT_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_SORTED_MAP))
#define GEE_IS_ABSTRACT_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_SORTED_MAP))
#define GEE_ABSTRACT_SORTED_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_SORTED_MAP, GeeAbstractSortedMapClass))

typedef struct _GeeAbstractSortedMap GeeAbstractSortedMap;
typedef struct _GeeAbstractSortedMapClass GeeAbstractSortedMapClass;
typedef struct _GeeAbstractSortedMapPrivate GeeAbstractSortedMapPrivate;

#define GEE_TYPE_READ_ONLY_MAP (gee_read_only_map_get_type ())
#define GEE_READ_ONLY_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMap))
#define GEE_READ_ONLY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMapClass))
#define GEE_IS_READ_ONLY_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_MAP))
#define GEE_IS_READ_ONLY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_MAP))
#define GEE_READ_ONLY_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMapClass))

typedef struct _GeeReadOnlyMap GeeReadOnlyMap;
typedef struct _GeeReadOnlyMapClass GeeReadOnlyMapClass;

#define GEE_TYPE_READ_ONLY_SORTED_MAP (gee_read_only_sorted_map_get_type ())
#define GEE_READ_ONLY_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMap))
#define GEE_READ_ONLY_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMapClass))
#define GEE_IS_READ_ONLY_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_SORTED_MAP))
#define GEE_IS_READ_ONLY_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_SORTED_MAP))
#define GEE_READ_ONLY_SORTED_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMapClass))

typedef struct _GeeReadOnlySortedMap GeeReadOnlySortedMap;
typedef struct _GeeReadOnlySortedMapClass GeeReadOnlySortedMapClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

typedef gboolean (*GeeForallFunc) (gpointer g, void* user_data);
typedef enum  {
	GEE_TRAVERSABLE_STREAM_YIELD,
	GEE_TRAVERSABLE_STREAM_CONTINUE,
	GEE_TRAVERSABLE_STREAM_END,
	GEE_TRAVERSABLE_STREAM_WAIT
} GeeTraversableStream;

typedef GeeTraversableStream (*GeeStreamFunc) (GeeTraversableStream state, GeeLazy* g, GeeLazy* * lazy, void* user_data);
struct _GeeIteratorIface {
	GTypeInterface parent_iface;
	gboolean (*next) (GeeIterator* self);
	gboolean (*has_next) (GeeIterator* self);
	gpointer (*get) (GeeIterator* self);
	void (*remove) (GeeIterator* self);
	gboolean (*get_valid) (GeeIterator* self);
	gboolean (*get_read_only) (GeeIterator* self);
};

typedef gpointer (*GeeFoldFunc) (gpointer g, gpointer a, void* user_data);
typedef gpointer (*GeeMapFunc) (gpointer g, void* user_data);
typedef gboolean (*GeePredicate) (gconstpointer g, void* user_data);
typedef GeeIterator* (*GeeFlatMapFunc) (gpointer g, void* user_data);
struct _GeeTraversableIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeTraversable* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeTraversable* self);
	GDestroyNotify (*get_g_destroy_func) (GeeTraversable* self);
	gboolean (*foreach) (GeeTraversable* self, GeeForallFunc f, void* f_target);
	GeeIterator* (*stream) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeStreamFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
	gpointer (*fold) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldFunc f, void* f_target, gpointer seed);
	GeeIterator* (*map) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeMapFunc f, void* f_target);
	GeeIterator* (*scan) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldFunc f, void* f_target, gpointer seed);
	GeeIterator* (*filter) (GeeTraversable* self, GeePredicate pred, void* pred_target, GDestroyNotify pred_target_destroy_notify);
	GeeIterator* (*chop) (GeeTraversable* self, gint offset, gint length);
	GType (*get_element_type) (GeeTraversable* self);
	GeeIterator* (*flat_map) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFlatMapFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
	GeeIterator** (*tee) (GeeTraversable* self, guint forks, int* result_length1);
	gpointer (*first_match) (GeeTraversable* self, GeePredicate pred, void* pred_target, GDestroyNotify pred_target_destroy_notify);
	gboolean (*any_match) (GeeTraversable* self, GeePredicate pred, void* pred_target, GDestroyNotify pred_target_destroy_notify);
	gboolean (*all_match) (GeeTraversable* self, GeePredicate pred, void* pred_target, GDestroyNotify pred_target_destroy_notify);
	gpointer (*max) (GeeTraversable* self, GCompareDataFunc compare, void* compare_target, GDestroyNotify compare_target_destroy_notify);
	gpointer (*min) (GeeTraversable* self, GCompareDataFunc compare, void* compare_target, GDestroyNotify compare_target_destroy_notify);
	GeeIterator* (*order_by) (GeeTraversable* self, GCompareDataFunc compare, void* compare_target, GDestroyNotify compare_target_destroy_notify);
};

struct _GeeIterableIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeIterable* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeIterable* self);
	GDestroyNotify (*get_g_destroy_func) (GeeIterable* self);
	GeeIterator* (*iterator) (GeeIterable* self);
};

typedef gpointer (*GeeFoldMapFunc) (gconstpointer k, gconstpointer v, gpointer a, void* user_data);
typedef gboolean (*GeeForallMapFunc) (gconstpointer k, gconstpointer v, void* user_data);
struct _GeeMapIteratorIface {
	GTypeInterface parent_iface;
	GType (*get_k_type) (GeeMapIterator* self);
	GBoxedCopyFunc (*get_k_dup_func) (GeeMapIterator* self);
	GDestroyNotify (*get_k_destroy_func) (GeeMapIterator* self);
	GType (*get_v_type) (GeeMapIterator* self);
	GBoxedCopyFunc (*get_v_dup_func) (GeeMapIterator* self);
	GDestroyNotify (*get_v_destroy_func) (GeeMapIterator* self);
	gboolean (*next) (GeeMapIterator* self);
	gboolean (*has_next) (GeeMapIterator* self);
	gpointer (*get_key) (GeeMapIterator* self);
	gpointer (*get_value) (GeeMapIterator* self);
	void (*set_value) (GeeMapIterator* self, gconstpointer value);
	void (*unset) (GeeMapIterator* self);
	gpointer (*fold) (GeeMapIterator* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldMapFunc f, void* f_target, gpointer seed);
	gboolean (*foreach) (GeeMapIterator* self, GeeForallMapFunc f, void* f_target);
	gboolean (*get_valid) (GeeMapIterator* self);
	gboolean (*get_mutable) (GeeMapIterator* self);
	gboolean (*get_read_only) (GeeMapIterator* self);
};

struct _GeeCollectionIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeCollection* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeCollection* self);
	GDestroyNotify (*get_g_destroy_func) (GeeCollection* self);
	gboolean (*contains) (GeeCollection* self, gconstpointer item);
	gboolean (*add) (GeeCollection* self, gconstpointer item);
	gboolean (*remove) (GeeCollection* self, gconstpointer item);
	void (*clear) (GeeCollection* self);
	gboolean (*add_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*contains_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*remove_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*retain_all) (GeeCollection* self, GeeCollection* collection);
	gpointer* (*to_array) (GeeCollection* self, int* result_length1);
	gint (*get_size) (GeeCollection* self);
	gboolean (*get_is_empty) (GeeCollection* self);
	gboolean (*get_read_only) (GeeCollection* self);
	GeeCollection* (*get_read_only_view) (GeeCollection* self);
	gboolean (*add_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*contains_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*remove_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*add_all_iterator) (GeeCollection* self, GeeIterator* iter);
	gboolean (*contains_all_iterator) (GeeCollection* self, GeeIterator* iter);
	gboolean (*remove_all_iterator) (GeeCollection* self, GeeIterator* iter);
};

struct _GeeSetIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeSet* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeSet* self);
	GDestroyNotify (*get_g_destroy_func) (GeeSet* self);
	GeeSet* (*get_read_only_view) (GeeSet* self);
};

struct _GeeMapIface {
	GTypeInterface parent_iface;
	GType (*get_k_type) (GeeMap* self);
	GBoxedCopyFunc (*get_k_dup_func) (GeeMap* self);
	GDestroyNotify (*get_k_destroy_func) (GeeMap* self);
	GType (*get_v_type) (GeeMap* self);
	GBoxedCopyFunc (*get_v_dup_func) (GeeMap* self);
	GDestroyNotify (*get_v_destroy_func) (GeeMap* self);
	gboolean (*has_key) (GeeMap* self, gconstpointer key);
	gboolean (*has) (GeeMap* self, gconstpointer key, gconstpointer value);
	gpointer (*get) (GeeMap* self, gconstpointer key);
	void (*set) (GeeMap* self, gconstpointer key, gconstpointer value);
	gboolean (*unset) (GeeMap* self, gconstpointer key, gpointer* value);
	void (*clear) (GeeMap* self);
	GeeMapIterator* (*map_iterator) (GeeMap* self);
	void (*set_all) (GeeMap* self, GeeMap* map);
	gboolean (*unset_all) (GeeMap* self, GeeMap* map);
	gboolean (*has_all) (GeeMap* self, GeeMap* map);
	gint (*get_size) (GeeMap* self);
	gboolean (*get_is_empty) (GeeMap* self);
	gboolean (*get_read_only) (GeeMap* self);
	GeeSet* (*get_keys) (GeeMap* self);
	GeeCollection* (*get_values) (GeeMap* self);
	GeeSet* (*get_entries) (GeeMap* self);
	GeeMap* (*get_read_only_view) (GeeMap* self);
};

struct _GeeAbstractMap {
	GObject parent_instance;
	GeeAbstractMapPrivate * priv;
};

struct _GeeAbstractMapClass {
	GObjectClass parent_class;
	gboolean (*has_key) (GeeAbstractMap* self, gconstpointer key);
	gboolean (*has) (GeeAbstractMap* self, gconstpointer key, gconstpointer value);
	gpointer (*get) (GeeAbstractMap* self, gconstpointer key);
	void (*set) (GeeAbstractMap* self, gconstpointer key, gconstpointer value);
	gboolean (*unset) (GeeAbstractMap* self, gconstpointer key, gpointer* value);
	GeeMapIterator* (*map_iterator) (GeeAbstractMap* self);
	void (*clear) (GeeAbstractMap* self);
	gboolean (*foreach) (GeeAbstractMap* self, GeeForallFunc f, void* f_target);
	GeeIterator* (*stream) (GeeAbstractMap* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeStreamFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
	void (*reserved0) (GeeAbstractMap* self);
	void (*reserved1) (GeeAbstractMap* self);
	void (*reserved2) (GeeAbstractMap* self);
	void (*reserved3) (GeeAbstractMap* self);
	void (*reserved4) (GeeAbstractMap* self);
	void (*reserved5) (GeeAbstractMap* self);
	void (*reserved6) (GeeAbstractMap* self);
	void (*reserved7) (GeeAbstractMap* self);
	void (*reserved8) (GeeAbstractMap* self);
	void (*reserved9) (GeeAbstractMap* self);
	gint (*get_size) (GeeAbstractMap* self);
	gboolean (*get_read_only) (GeeAbstractMap* self);
	GeeSet* (*get_keys) (GeeAbstractMap* self);
	GeeCollection* (*get_values) (GeeAbstractMap* self);
	GeeSet* (*get_entries) (GeeAbstractMap* self);
	GeeMap* (*get_read_only_view) (GeeAbstractMap* self);
};

struct _GeeSortedSetIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeSortedSet* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeSortedSet* self);
	GDestroyNotify (*get_g_destroy_func) (GeeSortedSet* self);
	gpointer (*first) (GeeSortedSet* self);
	gpointer (*last) (GeeSortedSet* self);
	GeeIterator* (*iterator_at) (GeeSortedSet* self, gconstpointer element);
	gpointer (*lower) (GeeSortedSet* self, gconstpointer element);
	gpointer (*higher) (GeeSortedSet* self, gconstpointer element);
	gpointer (*floor) (GeeSortedSet* self, gconstpointer element);
	gpointer (*ceil) (GeeSortedSet* self, gconstpointer element);
	GeeSortedSet* (*head_set) (GeeSortedSet* self, gconstpointer before);
	GeeSortedSet* (*tail_set) (GeeSortedSet* self, gconstpointer after);
	GeeSortedSet* (*sub_set) (GeeSortedSet* self, gconstpointer from, gconstpointer to);
	GeeSortedSet* (*get_read_only_view) (GeeSortedSet* self);
};

struct _GeeSortedMapIface {
	GTypeInterface parent_iface;
	GeeSortedMap* (*head_map) (GeeSortedMap* self, gconstpointer before);
	GeeSortedMap* (*tail_map) (GeeSortedMap* self, gconstpointer after);
	GeeSortedMap* (*sub_map) (GeeSortedMap* self, gconstpointer before, gconstpointer after);
	GeeSortedSet* (*get_ascending_keys) (GeeSortedMap* self);
	GeeSortedSet* (*get_ascending_entries) (GeeSortedMap* self);
	GeeSortedMap* (*get_read_only_view) (GeeSortedMap* self);
};

struct _GeeAbstractSortedMap {
	GeeAbstractMap parent_instance;
	GeeAbstractSortedMapPrivate * priv;
};

struct _GeeAbstractSortedMapClass {
	GeeAbstractMapClass parent_class;
	GeeSortedMap* (*head_map) (GeeAbstractSortedMap* self, gconstpointer before);
	GeeSortedMap* (*tail_map) (GeeAbstractSortedMap* self, gconstpointer after);
	GeeSortedMap* (*sub_map) (GeeAbstractSortedMap* self, gconstpointer before, gconstpointer after);
	void (*reserved0) (GeeAbstractSortedMap* self);
	void (*reserved1) (GeeAbstractSortedMap* self);
	void (*reserved2) (GeeAbstractSortedMap* self);
	void (*reserved3) (GeeAbstractSortedMap* self);
	void (*reserved4) (GeeAbstractSortedMap* self);
	void (*reserved5) (GeeAbstractSortedMap* self);
	void (*reserved6) (GeeAbstractSortedMap* self);
	void (*reserved7) (GeeAbstractSortedMap* self);
	void (*reserved8) (GeeAbstractSortedMap* self);
	void (*reserved9) (GeeAbstractSortedMap* self);
	GeeSortedSet* (*get_ascending_keys) (GeeAbstractSortedMap* self);
	GeeSortedSet* (*get_ascending_entries) (GeeAbstractSortedMap* self);
};

struct _GeeAbstractSortedMapPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
	GWeakRef _read_only_view;
};


static gpointer gee_abstract_sorted_map_parent_class = NULL;
static GeeSortedMapIface * gee_abstract_sorted_map_gee_sorted_map_parent_iface = NULL;

GType gee_traversable_stream_get_type (void) G_GNUC_CONST;
gpointer gee_lazy_ref (gpointer instance);
void gee_lazy_unref (gpointer instance);
GParamSpec* gee_param_spec_lazy (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void gee_value_set_lazy (GValue* value, gpointer v_object);
void gee_value_take_lazy (GValue* value, gpointer v_object);
gpointer gee_value_get_lazy (const GValue* value);
GType gee_lazy_get_type (void) G_GNUC_CONST;
GType gee_iterator_get_type (void) G_GNUC_CONST;
GType gee_traversable_get_type (void) G_GNUC_CONST;
GType gee_iterable_get_type (void) G_GNUC_CONST;
GType gee_map_iterator_get_type (void) G_GNUC_CONST;
GType gee_collection_get_type (void) G_GNUC_CONST;
GType gee_set_get_type (void) G_GNUC_CONST;
GType gee_map_entry_get_type (void) G_GNUC_CONST;
GType gee_map_get_type (void) G_GNUC_CONST;
GType gee_abstract_map_get_type (void) G_GNUC_CONST;
GType gee_sorted_set_get_type (void) G_GNUC_CONST;
GType gee_sorted_map_get_type (void) G_GNUC_CONST;
GType gee_abstract_sorted_map_get_type (void) G_GNUC_CONST;
#define GEE_ABSTRACT_SORTED_MAP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_ABSTRACT_SORTED_MAP, GeeAbstractSortedMapPrivate))
enum  {
	GEE_ABSTRACT_SORTED_MAP_0_PROPERTY,
	GEE_ABSTRACT_SORTED_MAP_K_TYPE,
	GEE_ABSTRACT_SORTED_MAP_K_DUP_FUNC,
	GEE_ABSTRACT_SORTED_MAP_K_DESTROY_FUNC,
	GEE_ABSTRACT_SORTED_MAP_V_TYPE,
	GEE_ABSTRACT_SORTED_MAP_V_DUP_FUNC,
	GEE_ABSTRACT_SORTED_MAP_V_DESTROY_FUNC,
	GEE_ABSTRACT_SORTED_MAP_ASCENDING_KEYS_PROPERTY,
	GEE_ABSTRACT_SORTED_MAP_ASCENDING_ENTRIES_PROPERTY,
	GEE_ABSTRACT_SORTED_MAP_READ_ONLY_VIEW_PROPERTY
};
GeeSortedMap* gee_abstract_sorted_map_head_map (GeeAbstractSortedMap* self, gconstpointer before);
static GeeSortedMap* gee_abstract_sorted_map_real_head_map (GeeAbstractSortedMap* self, gconstpointer before);
GeeSortedMap* gee_abstract_sorted_map_tail_map (GeeAbstractSortedMap* self, gconstpointer after);
static GeeSortedMap* gee_abstract_sorted_map_real_tail_map (GeeAbstractSortedMap* self, gconstpointer after);
GeeSortedMap* gee_abstract_sorted_map_sub_map (GeeAbstractSortedMap* self, gconstpointer before, gconstpointer after);
static GeeSortedMap* gee_abstract_sorted_map_real_sub_map (GeeAbstractSortedMap* self, gconstpointer before, gconstpointer after);
G_GNUC_INTERNAL void gee_abstract_sorted_map_reserved0 (GeeAbstractSortedMap* self);
static void gee_abstract_sorted_map_real_reserved0 (GeeAbstractSortedMap* self);
G_GNUC_INTERNAL void gee_abstract_sorted_map_reserved1 (GeeAbstractSortedMap* self);
static void gee_abstract_sorted_map_real_reserved1 (GeeAbstractSortedMap* self);
G_GNUC_INTERNAL void gee_abstract_sorted_map_reserved2 (GeeAbstractSortedMap* self);
static void gee_abstract_sorted_map_real_reserved2 (GeeAbstractSortedMap* self);
G_GNUC_INTERNAL void gee_abstract_sorted_map_reserved3 (GeeAbstractSortedMap* self);
static void gee_abstract_sorted_map_real_reserved3 (GeeAbstractSortedMap* self);
G_GNUC_INTERNAL void gee_abstract_sorted_map_reserved4 (GeeAbstractSortedMap* self);
static void gee_abstract_sorted_map_real_reserved4 (GeeAbstractSortedMap* self);
G_GNUC_INTERNAL void gee_abstract_sorted_map_reserved5 (GeeAbstractSortedMap* self);
static void gee_abstract_sorted_map_real_reserved5 (GeeAbstractSortedMap* self);
G_GNUC_INTERNAL void gee_abstract_sorted_map_reserved6 (GeeAbstractSortedMap* self);
static void gee_abstract_sorted_map_real_reserved6 (GeeAbstractSortedMap* self);
G_GNUC_INTERNAL void gee_abstract_sorted_map_reserved7 (GeeAbstractSortedMap* self);
static void gee_abstract_sorted_map_real_reserved7 (GeeAbstractSortedMap* self);
G_GNUC_INTERNAL void gee_abstract_sorted_map_reserved8 (GeeAbstractSortedMap* self);
static void gee_abstract_sorted_map_real_reserved8 (GeeAbstractSortedMap* self);
G_GNUC_INTERNAL void gee_abstract_sorted_map_reserved9 (GeeAbstractSortedMap* self);
static void gee_abstract_sorted_map_real_reserved9 (GeeAbstractSortedMap* self);
GeeAbstractSortedMap* gee_abstract_sorted_map_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func);
GeeAbstractMap* gee_abstract_map_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func);
GeeSortedSet* gee_abstract_sorted_map_get_ascending_keys (GeeAbstractSortedMap* self);
GeeSortedSet* gee_abstract_sorted_map_get_ascending_entries (GeeAbstractSortedMap* self);
G_GNUC_INTERNAL GeeReadOnlySortedMap* gee_read_only_sorted_map_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeMap* map);
G_GNUC_INTERNAL GeeReadOnlySortedMap* gee_read_only_sorted_map_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeMap* map);
G_GNUC_INTERNAL GType gee_read_only_map_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
G_GNUC_INTERNAL GType gee_read_only_sorted_map_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static GObject * gee_abstract_sorted_map_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void gee_abstract_sorted_map_finalize (GObject * obj);
GeeSortedMap* gee_sorted_map_get_read_only_view (GeeSortedMap* self);
static void _vala_gee_abstract_sorted_map_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_abstract_sorted_map_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


/**
 * {@inheritDoc}
 */
static GeeSortedMap* gee_abstract_sorted_map_real_head_map (GeeAbstractSortedMap* self, gconstpointer before) {
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_sorted_map_head_map'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


GeeSortedMap* gee_abstract_sorted_map_head_map (GeeAbstractSortedMap* self, gconstpointer before) {
	g_return_val_if_fail (self != NULL, NULL);
	return GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->head_map (self, before);
}


/**
 * {@inheritDoc}
 */
static GeeSortedMap* gee_abstract_sorted_map_real_tail_map (GeeAbstractSortedMap* self, gconstpointer after) {
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_sorted_map_tail_map'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


GeeSortedMap* gee_abstract_sorted_map_tail_map (GeeAbstractSortedMap* self, gconstpointer after) {
	g_return_val_if_fail (self != NULL, NULL);
	return GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->tail_map (self, after);
}


/**
 * {@inheritDoc}
 */
static GeeSortedMap* gee_abstract_sorted_map_real_sub_map (GeeAbstractSortedMap* self, gconstpointer before, gconstpointer after) {
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_sorted_map_sub_map'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


GeeSortedMap* gee_abstract_sorted_map_sub_map (GeeAbstractSortedMap* self, gconstpointer before, gconstpointer after) {
	g_return_val_if_fail (self != NULL, NULL);
	return GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->sub_map (self, before, after);
}


static void gee_abstract_sorted_map_real_reserved0 (GeeAbstractSortedMap* self) {
}


void gee_abstract_sorted_map_reserved0 (GeeAbstractSortedMap* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->reserved0 (self);
}


static void gee_abstract_sorted_map_real_reserved1 (GeeAbstractSortedMap* self) {
}


void gee_abstract_sorted_map_reserved1 (GeeAbstractSortedMap* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->reserved1 (self);
}


static void gee_abstract_sorted_map_real_reserved2 (GeeAbstractSortedMap* self) {
}


void gee_abstract_sorted_map_reserved2 (GeeAbstractSortedMap* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->reserved2 (self);
}


static void gee_abstract_sorted_map_real_reserved3 (GeeAbstractSortedMap* self) {
}


void gee_abstract_sorted_map_reserved3 (GeeAbstractSortedMap* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->reserved3 (self);
}


static void gee_abstract_sorted_map_real_reserved4 (GeeAbstractSortedMap* self) {
}


void gee_abstract_sorted_map_reserved4 (GeeAbstractSortedMap* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->reserved4 (self);
}


static void gee_abstract_sorted_map_real_reserved5 (GeeAbstractSortedMap* self) {
}


void gee_abstract_sorted_map_reserved5 (GeeAbstractSortedMap* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->reserved5 (self);
}


static void gee_abstract_sorted_map_real_reserved6 (GeeAbstractSortedMap* self) {
}


void gee_abstract_sorted_map_reserved6 (GeeAbstractSortedMap* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->reserved6 (self);
}


static void gee_abstract_sorted_map_real_reserved7 (GeeAbstractSortedMap* self) {
}


void gee_abstract_sorted_map_reserved7 (GeeAbstractSortedMap* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->reserved7 (self);
}


static void gee_abstract_sorted_map_real_reserved8 (GeeAbstractSortedMap* self) {
}


void gee_abstract_sorted_map_reserved8 (GeeAbstractSortedMap* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->reserved8 (self);
}


static void gee_abstract_sorted_map_real_reserved9 (GeeAbstractSortedMap* self) {
}


void gee_abstract_sorted_map_reserved9 (GeeAbstractSortedMap* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->reserved9 (self);
}


GeeAbstractSortedMap* gee_abstract_sorted_map_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func) {
	GeeAbstractSortedMap * self = NULL;
	self = (GeeAbstractSortedMap*) gee_abstract_map_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}


GeeSortedSet* gee_abstract_sorted_map_get_ascending_keys (GeeAbstractSortedMap* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->get_ascending_keys (self);
}


GeeSortedSet* gee_abstract_sorted_map_get_ascending_entries (GeeAbstractSortedMap* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return GEE_ABSTRACT_SORTED_MAP_GET_CLASS (self)->get_ascending_entries (self);
}


static GeeSortedMap* gee_abstract_sorted_map_real_get_read_only_view (GeeSortedMap* base) {
	GeeSortedMap* result;
	GeeAbstractSortedMap* self;
	GeeSortedMap* instance = NULL;
	GObject* _tmp0_;
	GeeSortedMap* _tmp1_;
	self = (GeeAbstractSortedMap*) base;
	_tmp0_ = g_weak_ref_get (&self->priv->_read_only_view);
	instance = G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_MAP, GeeSortedMap);
	_tmp1_ = instance;
	if (_tmp1_ == NULL) {
		GeeReadOnlySortedMap* _tmp2_;
		GeeSortedMap* _tmp3_;
		_tmp2_ = gee_read_only_sorted_map_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, (GeeMap*) self);
		_g_object_unref0 (instance);
		instance = (GeeSortedMap*) _tmp2_;
		_tmp3_ = instance;
		g_weak_ref_set (&self->priv->_read_only_view, (GObject*) _tmp3_);
	}
	result = instance;
	return result;
}


static GObject * gee_abstract_sorted_map_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GeeAbstractSortedMap * self;
	parent_class = G_OBJECT_CLASS (gee_abstract_sorted_map_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_ABSTRACT_SORTED_MAP, GeeAbstractSortedMap);
	g_weak_ref_clear (&self->priv->_read_only_view);
	g_weak_ref_init (&self->priv->_read_only_view, NULL);
	return obj;
}


static void gee_abstract_sorted_map_class_init (GeeAbstractSortedMapClass * klass) {
	gee_abstract_sorted_map_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeAbstractSortedMapPrivate));
	((GeeAbstractSortedMapClass *) klass)->head_map = (GeeSortedMap* (*) (GeeAbstractSortedMap *, gconstpointer)) gee_abstract_sorted_map_real_head_map;
	((GeeAbstractSortedMapClass *) klass)->tail_map = (GeeSortedMap* (*) (GeeAbstractSortedMap *, gconstpointer)) gee_abstract_sorted_map_real_tail_map;
	((GeeAbstractSortedMapClass *) klass)->sub_map = (GeeSortedMap* (*) (GeeAbstractSortedMap *, gconstpointer, gconstpointer)) gee_abstract_sorted_map_real_sub_map;
	((GeeAbstractSortedMapClass *) klass)->reserved0 = (void (*) (GeeAbstractSortedMap *)) gee_abstract_sorted_map_real_reserved0;
	((GeeAbstractSortedMapClass *) klass)->reserved1 = (void (*) (GeeAbstractSortedMap *)) gee_abstract_sorted_map_real_reserved1;
	((GeeAbstractSortedMapClass *) klass)->reserved2 = (void (*) (GeeAbstractSortedMap *)) gee_abstract_sorted_map_real_reserved2;
	((GeeAbstractSortedMapClass *) klass)->reserved3 = (void (*) (GeeAbstractSortedMap *)) gee_abstract_sorted_map_real_reserved3;
	((GeeAbstractSortedMapClass *) klass)->reserved4 = (void (*) (GeeAbstractSortedMap *)) gee_abstract_sorted_map_real_reserved4;
	((GeeAbstractSortedMapClass *) klass)->reserved5 = (void (*) (GeeAbstractSortedMap *)) gee_abstract_sorted_map_real_reserved5;
	((GeeAbstractSortedMapClass *) klass)->reserved6 = (void (*) (GeeAbstractSortedMap *)) gee_abstract_sorted_map_real_reserved6;
	((GeeAbstractSortedMapClass *) klass)->reserved7 = (void (*) (GeeAbstractSortedMap *)) gee_abstract_sorted_map_real_reserved7;
	((GeeAbstractSortedMapClass *) klass)->reserved8 = (void (*) (GeeAbstractSortedMap *)) gee_abstract_sorted_map_real_reserved8;
	((GeeAbstractSortedMapClass *) klass)->reserved9 = (void (*) (GeeAbstractSortedMap *)) gee_abstract_sorted_map_real_reserved9;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_abstract_sorted_map_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_abstract_sorted_map_set_property;
	G_OBJECT_CLASS (klass)->constructor = gee_abstract_sorted_map_constructor;
	G_OBJECT_CLASS (klass)->finalize = gee_abstract_sorted_map_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_SORTED_MAP_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_SORTED_MAP_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_SORTED_MAP_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_SORTED_MAP_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_SORTED_MAP_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_SORTED_MAP_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_SORTED_MAP_ASCENDING_KEYS_PROPERTY, g_param_spec_object ("ascending-keys", "ascending-keys", "ascending-keys", GEE_TYPE_SORTED_SET, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_SORTED_MAP_ASCENDING_ENTRIES_PROPERTY, g_param_spec_object ("ascending-entries", "ascending-entries", "ascending-entries", GEE_TYPE_SORTED_SET, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	/**
	 * The read-only view this map.
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_SORTED_MAP_READ_ONLY_VIEW_PROPERTY, g_param_spec_object ("read-only-view", "read-only-view", "read-only-view", GEE_TYPE_SORTED_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void gee_abstract_sorted_map_gee_sorted_map_interface_init (GeeSortedMapIface * iface) {
	gee_abstract_sorted_map_gee_sorted_map_parent_iface = g_type_interface_peek_parent (iface);
	iface->head_map = (GeeSortedMap* (*) (GeeSortedMap *, gconstpointer)) gee_abstract_sorted_map_head_map;
	iface->tail_map = (GeeSortedMap* (*) (GeeSortedMap *, gconstpointer)) gee_abstract_sorted_map_tail_map;
	iface->sub_map = (GeeSortedMap* (*) (GeeSortedMap *, gconstpointer, gconstpointer)) gee_abstract_sorted_map_sub_map;
	iface->get_ascending_keys = (GeeSortedSet* (*) (GeeSortedMap *)) gee_abstract_sorted_map_get_ascending_keys;
	iface->get_ascending_entries = (GeeSortedSet* (*) (GeeSortedMap *)) gee_abstract_sorted_map_get_ascending_entries;
	iface->get_read_only_view = gee_abstract_sorted_map_real_get_read_only_view;
}


static void gee_abstract_sorted_map_instance_init (GeeAbstractSortedMap * self) {
	self->priv = GEE_ABSTRACT_SORTED_MAP_GET_PRIVATE (self);
}


static void gee_abstract_sorted_map_finalize (GObject * obj) {
	GeeAbstractSortedMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_ABSTRACT_SORTED_MAP, GeeAbstractSortedMap);
	g_weak_ref_clear (&self->priv->_read_only_view);
	G_OBJECT_CLASS (gee_abstract_sorted_map_parent_class)->finalize (obj);
}


GType gee_abstract_sorted_map_get_type (void) {
	static volatile gsize gee_abstract_sorted_map_type_id__volatile = 0;
	if (g_once_init_enter (&gee_abstract_sorted_map_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeAbstractSortedMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_abstract_sorted_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeAbstractSortedMap), 0, (GInstanceInitFunc) gee_abstract_sorted_map_instance_init, NULL };
		static const GInterfaceInfo gee_sorted_map_info = { (GInterfaceInitFunc) gee_abstract_sorted_map_gee_sorted_map_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gee_abstract_sorted_map_type_id;
		gee_abstract_sorted_map_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_MAP, "GeeAbstractSortedMap", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_type_add_interface_static (gee_abstract_sorted_map_type_id, GEE_TYPE_SORTED_MAP, &gee_sorted_map_info);
		g_once_init_leave (&gee_abstract_sorted_map_type_id__volatile, gee_abstract_sorted_map_type_id);
	}
	return gee_abstract_sorted_map_type_id__volatile;
}


static void _vala_gee_abstract_sorted_map_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeAbstractSortedMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_ABSTRACT_SORTED_MAP, GeeAbstractSortedMap);
	switch (property_id) {
		case GEE_ABSTRACT_SORTED_MAP_READ_ONLY_VIEW_PROPERTY:
		g_value_take_object (value, gee_sorted_map_get_read_only_view ((GeeSortedMap*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_abstract_sorted_map_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeAbstractSortedMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_ABSTRACT_SORTED_MAP, GeeAbstractSortedMap);
	switch (property_id) {
		case GEE_ABSTRACT_SORTED_MAP_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_ABSTRACT_SORTED_MAP_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_ABSTRACT_SORTED_MAP_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_ABSTRACT_SORTED_MAP_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_ABSTRACT_SORTED_MAP_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_ABSTRACT_SORTED_MAP_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}



